// image.cpp : Defines the entry point for the console application
// bmp save routines.  image merge routines. jpg save routines
// uses jpeg-6b driver.
// Copyright 2006-2010 Sensoray Corp Inc.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define MAX_PATH 260
#include <time.h>
//#include <wchar.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#ifdef __cplusplus
extern "C" {
#endif
#ifdef LINUX_JPEG_SUPPORT
#include "jpeg-6b/jpeglib.h"
#include "jpeg-6b/cdjpeg.h"       /* Common decls for cjpeg/djpeg applications */
#include "jpeg-6b/jversion.h"     /* for version message */
#endif
#ifdef __cplusplus
}
#endif

#include "image.h"
#include "ovlmid.h"

/*lint -e732 -e715 -e529 -e734 */

// image.cpp
// routines for image conversion 

void reverse_rgb_line( unsigned char *pline, int length);



/** Converts 8-bit RGB to 6-5-5-bit YCrCb using the BT601 color space definition.
 *
 * Y  =  0.2568*Rd + 0.5041*Gd + 0.0979*Bd + 16 
 * Cr =  0.4392*Rd - 0.3678*Gd - 0.0714*Bd + 128
 * Cb = -0.1482*Rd - 0.2910*Gd + 0.4392*Bd + 128
 *
 * Using integer math.
 *
 * Y  = ( 16829*Rd + 33039*Gd +  6416*Bd + 1048576 ) / 65536
 * Cr = ( 28784*Rd - 24103*Gd -  4681*Bd + 8388608 ) / 65536
 * Cb = ( -9714*Rd - 19071*Gd + 28784*Bd + 8388608 ) / 65536
 *
 *  @param R red value
 *  @param G green value
 *  @param B blue value
 *  @return YCrCb triplet short
 */
unsigned short RGB2YCrCb16_BT601( int R, int G, int B )
{
    int Y  = ( 16829*R + 33039*G +  6416*B + 1048576 ) >> ( 16 + 2 );
    int Cr = ( 28784*R - 24103*G -  4681*B + 8388608 ) >> ( 16 + 3 );
    int Cb = ( -9714*R - 19071*G + 28784*B + 8388608 ) >> ( 16 + 3 );
    
    Y  = Y  > ( 235 >> 2 ) ? ( 235 >> 2 ) : Y;
    Cr = Cr > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cr;
    Cb = Cb > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cb;
    
    Y  = Y  < ( 16 >> 2 ) ? ( 16 >> 2 ) : Y;
    Cr = Cr < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cr;
    Cb = Cb < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cb;
    return ( Y << 10 ) + ( Cr << 5 ) + Cb;
}

/** Converts 8-RGB to 10-bit YCrCb using
 * the BT709 color space definition.
 * Y  =  0.7303*R + 2.4569*G + 0.2480*B + 64
 * Cr =  1.7569*R - 1.5958*G - 0.1611*B + 512
 * Cb = -0.4026*R - 1.3543*G + 1.7569*B + 512
 *
 * Using integer math.
 *
 * Y  = ( 11966*R + 40254*G +  4064*B + 1048576 ) / 16384 @ 10-bits
 * Cr = ( 28784*R - 26145*G -  2639*B + 8388608 ) / 16384 @ 10-bits
 * Cb = ( -6596*R - 22189*G + 28784*B + 8388608 ) / 16384 @ 10-bits
 *
 *  @param R red value
 *  @param G green value
 *  @param B blue value
 *  @return YCrCb triplet short
 */
unsigned short RGB2YCrCb16_BT709( int R, int G, int B )
{
    int Y  = ( 11966*R + 40254*G +  4064*B + 1048576 ) >> ( 14 + 4 ); // 10-4 = 6-bits
    int Cr = ( 28784*R - 26145*G -  2639*B + 8388608 ) >> ( 14 + 5 ); // 10-5 = 5-bits
    int Cb = ( -6596*R - 22189*G + 28784*B + 8388608 ) >> ( 14 + 5 ); // 10-5 = 5-bits
    
    Y  = Y  > ( 235 >> 2 ) ? ( 235 >> 2 ) : Y;
    Cr = Cr > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cr;
    Cb = Cb > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cb;
    
    Y  = Y  < ( 16 >> 2 ) ? ( 16 >> 2 ) : Y;
    Cr = Cr < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cr;
    Cb = Cb < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cb;
    return ( Y << 10 ) + ( Cr << 5 ) + Cb;
}

/** Converts 8-bit YCrCb to 8-bit RGB using the BT601 color space definition
 *
 * R = 1.16438356164384*(Y - 16) + 1.59602678571429 *(Cr - 128)
 * G = 1.16438356164384*(Y - 16) - 0.812967647237771*(Cr - 128) - 0.391762290094914*(Cb - 128)
 * B = 1.16438356164384*(Y - 16) + 2.01723214285714 *(Cb - 128)
 *
 * Using integer math.
 *
 * R = ( 76309*(Y-16) + 104597*(Cr-128)                  ) / 65536
 * G = ( 76309*(Y-16) -  53279*(Cr-128) - 25675*(Cb-128) ) / 65536
 * B = ( 76309*(Y-16) + 132201*(Cb-128)                  ) / 65536
 * 
 * @param Y : y  value (unsigned char in a long integer)
 * @param Cr: Cr value (unsigned char in a long integer)
 * @param Cb: Cb value (unsigned char in a long integer)
 * @param *pR: returned R value (unsigned char)
 * @param *pG: returned G value (unsigned char)
 * @param *pB: returned B value (unsigned char)
*/
void YCrCb2RGB_8bit_BT601( int Y, int Cr, int Cb, unsigned char *pR, unsigned char *pG, unsigned char *pB)
{
    int R,G,B;
   
    Y = Y - 16;
    Cr = Cr - 128;
    Cb = Cb - 128;
    
    // These are the numbers from 10-bit YCrCb to 10-bit RGB for BT.601
    //R = ((76533*Y) + (104936*Cr)               ) >> 16;
    //G = ((76533*Y) - ( 53451*Cr) - ( 25703*Cb) ) >> 16;
    //B = ((76533*Y)               + (132677*Cb) ) >> 16;

    R = ((76309*Y) + (104597*Cr)               ) >> 16;
    G = ((76309*Y) - ( 53279*Cr) - ( 25675*Cb) ) >> 16;
    B = ((76309*Y)               + (132201*Cb) ) >> 16;
    
    // Even with proper conversion, some values still need clipping.
    if (R > 255) R = 255;
    if (G > 255) G = 255;
    if (B > 255) B = 255;
    if (R < 0) R = 0;
    if (G < 0) G = 0;
    if (B < 0) B = 0;
    
    *pR = (BYTE)R;
    *pG = (BYTE)G;
    *pB = (BYTE)B;
    return;
}


/** Converts 8-bit YCrCb to 8-bit RGB using the BT709 color space definition
 *
 * R = 1.16438356164384*(Y - 16) + 1.79274107142857*(Cr - 128)
 * G = 1.16438356164384*(Y - 16) - 0.532909328559444*(Cr - 128) - 0.21324861427373*(Cb - 128)
 * B = 1.16438356164384*(Y - 16) + 2.11240178571429*(Cb - 128)
 *
 * Using integer math.
 *
 * R = ( 76309*(Y-16) + 117489*(Cr-128)                   ) / 65536
 * G = ( 76309*(Y-16) -  34925*(Cr-128) -  13975*(Cb-128) ) / 65536
 * B = ( 76309*(Y-16)                   + 138438*(Cb-128) ) / 65536
 * 
 * @param Y : y  value (unsigned char in a long integer)
 * @param Cr: Cr value (unsigned char in a long integer)
 * @param Cb: Cb value (unsigned char in a long integer)
 * @param *pR: returned R value (BYTE)
 * @param *pG: returned G value (BYTE)
 * @param *pB: returned B value (BYTE)
*/
void YCrCb2RGB_8bit_BT709( int Y, int Cr, int Cb, unsigned char *pR, unsigned char *pG, unsigned char *pB)
{
    int R,G,B;
   
    Y = Y - 16;
    Cr = Cr - 128;
    Cb = Cb - 128;
    
    // These are the numbers from 10-bit YCrCb to 10-bit RGB for BT.601
    //R = ((76533*Y) + (104936*Cr)               ) >> 16;
    //G = ((76533*Y) - ( 53451*Cr) - ( 25703*Cb) ) >> 16;
    //B = ((76533*Y)               + (132677*Cb) ) >> 16;

    R = ((76309*Y) + (117489*Cr)               ) >> 16;
    G = ((76309*Y) - ( 34925*Cr) - ( 13975*Cb) ) >> 16;
    B = ((76309*Y)               + (138438*Cb) ) >> 16;
    
    // Even with proper conversion, some values still need clipping.
    if (R > 255) R = 255;
    if (G > 255) G = 255;
    if (B > 255) B = 255;
    if (R < 0) R = 0;
    if (G < 0) G = 0;
    if (B < 0) B = 0;
    
    *pR = (BYTE)R;
    *pG = (BYTE)G;
    *pB = (BYTE)B;
    return;
}

/** Invert field zero
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1
 *  @param h1 height of field 1
 *  @param w  width of fields( full width in bytes)
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int inv_fieldzero(unsigned char *newf, int size,  unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH)
{
    int i;
    int j;
    j = h0-1;
    for( i=0; i< h0; i++, j-=1)
    {
        memcpy( newf + j*w,     &f0[i*w], w);
        // reverse BGR --> RGB
        reverse_rgb_line(newf + j*w, w);
    }
    return 0;
}

/** duplicate fields
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1
 *  @param h1 height of field 1
 *  @param w  width of fields( full width in bytes)
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int dup_fields( unsigned char *newf, int size,  unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH)
{
    int i;
    int j;
    
    if (h1 == 0) {
        // Progressive screen
        memcpy( newf, &f0[0], w*h0);
        return 0;
    } 

    for( i=0,j=0; i< h0; i++, j+=2)
    {
        memcpy( newf + j*w,     &f0[i*w], w);
        memcpy( newf + (j+1)*w, &f0[i*w], w);
    }
    return 0;
}


/** merge fields
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1 (unused)
 *  @param h1 height of field 1  (unused)
 *  @param w  width of fields in bytes
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int merge_fields( unsigned char *newf, int size,  unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH)
{
    int i;
    int j;
    int h;
    
    if (h1 == 0) {
        // Progressive screen
        memcpy( newf, &f0[0], w*h0);
        return 0;
    } 
    
    h = (h0 < h1) ? h0 : h1;

    // else Interlaced screen
    for( i=0,j=0; i< h; i++, j+=2)
    {
        memcpy( newf + w*j,     &f0[i*w], w);
        memcpy( newf + w*(j+1), &f1[i*w], w);
    }
    return 0;
}


/** interpolate fields
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1 
 *  @param h1 height of field 1  
 *  @param w  width of fields in bytes
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int inter_fields( unsigned char *newf, int size,  unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH)
{
    int i;
    int j;
    int h;
    // fill out the fields( every other field)
    if (h1 == 0) {
        // Progressive screen
        memcpy( newf, &f0[0], w*h0);
        return 0;
    } 
    // else Interlaced

    h = (h0 < h1) ? h0 : h1;
    for( i=0, j=0; i< h; i++, j+=2)
    {
        memcpy( newf + j*w, &f0[i*w], w);
    }

    for ( i= 1; (i+1) < 2* h; i+=2)
    {
        for (j=0; j< w; j++)
        {
            *(newf+i*w + j) = (*(newf + (i-1)*w + j) + *(newf + (i+1)*w + j) ) / 2;
        }
    }
    return 0;  
}//lint !e550



/** reverse the RGB ordering in a line.  Used for the JPEG library.  pline is changed by this function
 *  @param pline pointer to line
 *  @param length length of line
 *  @return none
 */
void reverse_rgb_line( unsigned char *pline, int length)
{
    int i;
    unsigned char tmp;
    
    for( i=0; i< length ;i+=3)
    {
        tmp = *(pline+i);
        *(pline+i) = *(pline+2+i);
        *(pline+2+i) = tmp;
    }
    return;
}

#ifdef LINUX_JPEG_SUPPORT
/** converts image buffer to JPG file specified
* @param image_buffer is a pointer to the image
* @param outfilename is the full path name of the resulting file
* @param info is the image info structure
* @param qual quality of jpeg.  25-100 where 100 is highest quality(largest file size)
* @return 0 on success, -ve on fail
*/
int image_to_jpg( const unsigned char *image, const wchar_t* outfilename, image_info_t *info, int qual )
{
    struct jpeg_compress_struct cinfo;
    struct jpeg_error_mgr jerr;
    FILE *fileout;
    int num_scanlines;
    int i;
    unsigned char *pline;
    char name[MAX_PATH];

    // open the file
    if( wcsstr(outfilename, ".jpg") == NULL)
        snprintf(name, MAX_PATH, "%s.%s", outfilename, L"jpg");
    else
        strncpy(name, outfilename, MAX_PATH);


    //OutputDebugString("image to jpg: ");
    //OutputDebugString(name);
    // open output file
    fileout = fopen(name, "wb");


    if( fileout == NULL)
        return -1;
    
    /* Initialize the JPEG compression object with default error handling. */
    cinfo.err = jpeg_std_error(&jerr);
    jpeg_create_compress(&cinfo);

    // set RGB space
    cinfo.in_color_space = JCS_RGB; 
    
    // set the defaults
    jpeg_set_defaults(&cinfo);
    jpeg_default_colorspace(&cinfo);

    // set the destination out file
    jpeg_stdio_dest( &cinfo, fileout);  

    // initialize the jpeg params
    cinfo.image_height = info->height;
    cinfo.image_width = info->width;
    cinfo.input_components = 3;
    cinfo.in_color_space = (J_COLOR_SPACE) 2;
    cinfo.is_decompressor = 0;
    //cinfo.global_state = 100;
    cinfo.input_gamma = 1;
    cinfo.data_precision = 8;
    //cinfo.num_components = 3;
    //cinfo.jpeg_color_space =3;
    if( qual >100 )
        qual =100;
    else if( qual < 25)
        qual = 25;

    jpeg_set_quality( &cinfo, qual, TRUE);

    // start the Library compression
    jpeg_start_compress( &cinfo, TRUE);

    // start the compression
    num_scanlines = 1;  // one line at a time
    
    // write out all the lines
    pline = (unsigned char *) malloc( cinfo.image_width * 3 );
    if (pline == NULL)
        return -1;

#ifndef MID2250_BUILD
    for (i=0; i < (int) cinfo.image_height; i++)  // regular order
#else
    for (i= cinfo.image_height-1; i>=0; i--)  // reverse order
#endif
    {
        // reverse RGB --> BGR
        // not efficient, but safer to copy the line before reversing.
        memcpy( pline, &image[i*cinfo.image_width *3], cinfo.image_width * 3);
#if defined (MID2250_BUILD)
        reverse_rgb_line( pline, cinfo.image_width*3);
#endif
        (void) jpeg_write_scanlines(&cinfo, &pline, num_scanlines);
#ifndef SENSORAY_OS_LINUX
        // important, sleep while writing scanlines to free CPU for other tasks
        if( (i%20) == 0) {
            Sleep(1);
		}
#endif
    }
    free( pline);
    
    // finish up
    jpeg_finish_compress( &cinfo);
    jpeg_destroy_compress( &cinfo);

    // close the output file
    fclose( fileout);
    return 0;
}

#endif


/** saves an image to bmp file
 *  @param filename is the full file path to save( minus the extension)
 *  @param *image, pointer to image data in Y Cr Cb format
 *  @param type is the type( 0-- BMP,  1--ppm)
 *  @return 0 on success, -1 if file invalid
*/
int save_image_uncompressed(const unsigned char *image, const char *szFilename, image_info_t *info, int type)
{
    char ppmheader[256];
    char name[MAX_PATH];
    FILE *fptr;
    int val;
    int rlen; // row length or number of columns( bytes== pixels * bytes per pixel)
    int clen; // column length or number of rows
    int bpp; // bytes per pixel! not bits
    int i;
    
    rlen = info->stride;
    clen = info->height;
    bpp = info->stride / info->width;
    
    // open the file
    if (type == 0) 
      snprintf(name, MAX_PATH, "%s.%s", szFilename, "bmp");
    else
	    snprintf(name, MAX_PATH, "%s.%s", szFilename, "ppm");

    fptr = fopen(name, "wb");

    if (fptr == NULL)
        return -1;

    if (type == 1)
    {
        sprintf( ppmheader, "P6\n#ppm image\n%d %d\n255\n", info->width, info->height);
        fwrite( ppmheader, 1, strlen(ppmheader), fptr);
        // write out the rows
        for ( i=0; i< clen; i++)
        {
            fwrite( &image[rlen*i], 1, rlen, fptr);
        }
    }
    else
    {
        val = 0x4d42;                   fwrite(&val,1,2,fptr); //bmp signature
        val = (rlen * clen / bpp) + 54; fwrite(&val,1,4,fptr); //size of bmp file
        val = 0;                        fwrite(&val,1,2,fptr); //must be 0
        val = 0;                        fwrite(&val,1,2,fptr); //must be 0
        val = 54;                       fwrite(&val,1,4,fptr); //offset to image start
        val = 40;                       fwrite(&val,1,4,fptr); //must be 40
        val = rlen / bpp;               fwrite(&val,1,4,fptr); //image width in pixels
        val = clen ;                    fwrite(&val,1,4,fptr); //image height in pixels
        val = 1;                        fwrite(&val,1,2,fptr); //must be 1
        val = bpp * 8;                  fwrite(&val,1,2,fptr); //bits per pixel
        val = 0;                        fwrite(&val,1,4,fptr); //compression (none)
        val = (rlen * clen / bpp);      fwrite(&val,1,4,fptr); //size of image data
        val = 2835;                     fwrite(&val,1,4,fptr); //horizontal pixels/m
        val = 2835;                     fwrite(&val,1,4,fptr); //vertical pixels/m
        val = 0;                        fwrite(&val,1,4,fptr); //colors in image, or 0
        val = 0;                        fwrite(&val,1,4,fptr); //important colors,or 0
#ifndef MID2250_BUILD
        // BMP reverse the image order. for 2246, retrieves in reverse order
        for (i= clen-1; i>= 0; i--)  //rows
        {
#if defined (MID2246_BUILD)
           // BMP -DOES- reverse RGB --> BGR
			// DA: 3/23/2007.  Warning.  This violates the const restriction on image
            reverse_rgb_line((unsigned char *)&image[i*rlen], rlen);
#endif

            fwrite( &image[i*rlen], 1, rlen, fptr);
        }
#else
        for (i= 0; i<clen; i++)  //rows
        {
            fwrite( &image[i*rlen], 1, rlen, fptr);
        }
#endif

    }
    fclose( fptr);
    return 0;
}

/** Converts 8-bit YCrCb image to 8-bit RGB image in memory using BT.601 color space
 *  @param in pointer to input image
 *  @param out pointer to output image
 *  @param height height of image
 *  @param width width of image
 *  @return 0 on success
 */
int YCrCb2RGB_BT601( const unsigned char *in, unsigned char *out, int height, int width)
{
    int i;
    int k;
    unsigned char y,Cr,Cb;
    unsigned char r,g,b;

    // convert the image using BT601
    for ( i = 0; i< height; i++) {
        
        for (k=0 ; k < (width *3); k+=3)
        {
            y  = in[i*width*3+k  ];
            Cr = in[i*width*3+k+1];
            Cb = in[i*width*3+k+2];
            YCrCb2RGB_8bit_BT601(y,Cr,Cb,&r,&g,&b);
            out[i*width*3+k  ] = r;
            out[i*width*3+k+1] = g;
            out[i*width*3+k+2] = b;
        }
    }
    return 0; 
}


/** Converts 8-bit YCrCb image to 8-bit RGB image in memory using BT.709 color space
 *  @param in pointer to input image
 *  @param out pointer to output image
 *  @param height height of image
 *  @param width width of image
 *  @return 0 on success
 */
int YCrCb2RGB_BT709( const unsigned char *in, unsigned char *out, int height, int width)
{
    int i;
    int k;
    unsigned char y,Cr,Cb;
    unsigned char r,g,b;

    // convert the image using BT709
    for ( i = 0; i< height; i++) {
        
        for (k=0 ; k < (width *3); k+=3)
        {
            y  = in[i*width*3+k  ];
            Cr = in[i*width*3+k+1];
            Cb = in[i*width*3+k+2];
            YCrCb2RGB_8bit_BT709(y,Cr,Cb,&r,&g,&b);
            out[i*width*3+k  ] = r;
            out[i*width*3+k+1] = g;
            out[i*width*3+k+2] = b;
        }
    }
    return 0; 
}
