// driver interaction/raw functions
// 
// begin DRIVER DEVICE helper functions 
// begin FPGA driver specific functions( related to m_handle)
// ALL DeviceIoControl functions must be put in this module!
/*lint -e713 */ //Loss of precision (assignment) (unsigned int to int)
/*lint -e734 */ //Loss of precision (assignment) (31 bits to 7 bits)
/*lint -e732 */ //Loss of sign

//NOTE: 2256 currently shares the same ioctls as the 2226
//TODO: map the calls if the ioctls change. For now, ioctl2226.h
// 

#include "drivermid.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "s2226ioctl.h"
#include <sys/ioctl.h>

//int s2226_write_addr_data(HANDEL handle, S2226_REG_OVAUPD, 0x00)

// writes data to address
int s2226_write_data(int handle, int addr, unsigned short value)
{
  struct io_reg p;
  int rc;
  p.addr = addr;
  p.val = value;
  rc = ioctl(handle, S2226_VIDIOC_FPGA_WR, &p);
  return rc;
}

// writes burst of data to address
int s2226_write_addr_data_burst(int handle, int addr, unsigned char *data, int len, int bFast)
{
    int rc;
    struct io_burst p;
    if (len > (512-14))
      return -1;
    p.addr = addr;
    memcpy(p.data, data, len);
    p.len = len;
    rc = ioctl(handle, bFast ? S2226_VIDIOC_FPGA_WR_BURST_FAST : S2226_VIDIOC_FPGA_WR_BURST, &p);
    return rc;
}

// writes to FPGA interleaved address, data format
int s2226_write_addr_data_interleaved(int handle, int addr, unsigned char *data, int len)
{
    int rc;
    struct io_burst p;
    if (len > (512-14))
      return -1;
    p.addr = addr;
    memcpy(p.data, data, len);
    p.len = len;
    rc = ioctl(handle, S2226_VIDIOC_FPGA_WR_ADDRDATA, &p);
    return rc;
}

int s2226_read_addr_data(int handle, int addr, unsigned short *value)
{
  struct io_reg p;
  int rc;
  p.addr = addr;
  p.val = 0xcd;
  rc = ioctl(handle, S2226_VIDIOC_FPGA_RD, &p);
  *value = p.val;
  return rc;
}

int s2226_read_addr_data_burst(int handle, int addr, unsigned char *data, int len)
{
    int rc;
    struct io_burst p;
    if (len > 508)
      return -1;
    p.addr = addr;
    p.len = len;
    rc = ioctl(handle, S2226_VIDIOC_FPGA_RD_BURST, &p);
    memcpy(data, p.data, len);
    return rc;
}

/** read burst
 *  @param reg_addr, address to read from
 *  @param buf, pointer to buffer
 *  @param bytecount number of bytes desired
 *  @return number of bytes read
 */
int s2226_BurstReadTransfer( MYHDEVICE drv_handle, int addr, void *buf, int bytecount )
{
    unsigned char *pBuffer = (unsigned char *) buf;
    //char *pBuffer = (char *) pbuf;
    int count = 0;
    unsigned char rbuf[512];
    int i;
    //CriticalSectionBegin( CriticalSection_DEVICE);
#define BURST_READ_SIZE (512-4) // must be a multiple of 4.  Can not be larger than 512 - 4( needed for return buffer header)
    while (count < bytecount) {
        int rlen;
        rlen = bytecount-count; // number of bytes left
        // we require 4 bytes read for every 3 bytes of data
        rlen = rlen * 4 /3;
        // should be multiple of 4;
        rlen += rlen % 4;
        // limit to BURST_READ_SIZE per read
        rlen = rlen > BURST_READ_SIZE ? BURST_READ_SIZE : rlen;
        s2226_read_addr_data_burst( drv_handle, addr, rbuf, rlen);
        for (i = 0; i < rlen; i+=4) {
            // Data read back in this order Cr Cb _ Y where _ is a blank field.
            *pBuffer++ = rbuf[i+3]; // Y
            *pBuffer++ = rbuf[i+1]; // Cb
            *pBuffer++ = rbuf[i]; // Cr
            count += 3; // we skip every 4th byte
            if (count >= bytecount)
                break;
        }
    }
    //CriticalSectionEnd( CriticalSection_DEVICE);
    return bytecount;
}

void s2226_BurstWriteTransfer( MYHDEVICE drv_handle, int addr, void *pbuf, int bytecount, int type )
{
// Can not be larger than 256 - 14( needed for header plus checksum), should be multiple of 2
// USB endpoint is 512, but unfortunately FPGA commands length field is currently only
// 1 byte for a maximum command of 256.  This issue is currently being investigated without breaking
// backward compatibility.
#define BURST_WRITE_SIZE (256-14) 
    unsigned char *pBuffer = (unsigned char *) pbuf;
    unsigned char wbuffer[BURST_WRITE_SIZE];
    int wlen;
    int i;

    while (bytecount>0) {
        wlen = BURST_WRITE_SIZE;
        if (wlen == bytecount)
            wlen += wlen % 2;
        for (i = 0; i < wlen; i+=2) {
            wbuffer[i] = 0;
            wbuffer[i+1] = *pBuffer++;
        }
        // no unpack
        s2226_write_addr_data_burst(drv_handle, addr, wbuffer, wlen, 0);
        bytecount-= wlen/2;
    }
}


void s2226_BurstWriteTransferFast( MYHDEVICE drv_handle, int addr, void *pbuf, int bytecount, int type )
{
// Can not be larger than 512 - 14( needed for header plus checksum), should be multiple of 2
#define BURST_WRITE_SIZE_FAST (512-14) 
    unsigned char *pBuffer = (unsigned char *) pbuf;
    unsigned char wbuffer[BURST_WRITE_SIZE_FAST];
    int wlen;
    int i;

    while (bytecount>0) {
        wlen = BURST_WRITE_SIZE_FAST;
        for (i = 0; i < wlen; i++) {
            wbuffer[i] = *pBuffer++;
        }
        // with unpack
        s2226_write_addr_data_burst(drv_handle, addr, wbuffer, wlen, 1);
        bytecount-= wlen/2;
    }
}

