// please see fpga2226.cpp for function descriptions

#ifndef _FPGA2226_H
#define _FPGA2226_H

//#include "types.h"  // HDEVICE
//#include "mid2226const.h"
#include "drivermid.h"

// These are the values read from the S2226_REG_BOARDID for various board revisions
//#define S2226_BOARD_0  0
//#define S2226_BOARD_A  1
//#define S2226_BOARD_B  2

//////////////////////////////////////////////////////////////////////////
//  FPGA REGISTERS
//////////////////////////////////////////////////////////////////////////

// Direct access registers -------------------------------



  // Video Control and Status 
#define S2226_REG_SDICLKSEL     (0x000<<1)
#define S2226_REG_SDIIN         (0x001<<1)
#define S2226_REG_SDIOUT        (0x002<<1)
#define S2226_REG_VIDCFG        (0x003<<1)
#define S2226_REG_VIDSTAT       (0x004<<1)
#define S2226_REG_VINSEL        (0x005<<1)
#define S2226_REG_AUDCFG        (0x006<<1)
#define S2226_REG_RESETCNT      (0x007<<1)
#define S2226_REG_VICLKPHASE    (0x008<<1)
#define S2226_REG_PLL_DATA      (0x009<<1)
#define S2226_REG_PLL_CTRL      (0x00A<<1)

// Reset and Interrupt status/control
#define S2226_REG_RESETCTRL     (0x010<<1)
#define S2226_REG_INTENAB       (0x011<<1)
#define S2226_REG_INTSTAT       (0x012<<1)

// Input Embedded Audio control
#define S2226_REG_AUD_INPK_CTL  (0x015<<1)

// Audio Bus for updating timing parameters
#define S2226_REG_AUD_BUS_CTL   (0x016<<1)
#define S2226_REG_AUD_BUS_ADR   (0x017<<1)
#define S2226_REG_AUD_BUS_D0    (0x018<<1)
#define S2226_REG_AUD_BUS_D1    (0x019<<1)

// Output Embedded Audio Packet Control
#define S2226_REG_AUD_OUTPK_CTL (0x01A<<1)

// Output Embedded Audio Packet info
#define S2226_REG_AUD_OUTPK_INF (0x01B<<1)
#define S2226_REG_AUD_CSBOUT_A  (0x01C<<1)
#define S2226_REG_AUD_CSBOUT_B  (0x01D<<1)

// Output Embedded Audio Tone generation
#define S2226_REG_AUD_TONE_A    (0x01E<<1)
#define S2226_REG_AUD_TONE_B    (0x01F<<1)

// Miscellaneous.
#define S2226_REG_LEDDATA       (0x020<<1)    // r/w  Led control signals
#define S2226_REG_BOARDID       (0x021<<1)    // r   [15..11] Indicates board rev, [10..0] Indicates FPGA rev
#define S2226_REG_DEBUG         (0x022<<1)    // w    Misc
#define S2226_REG_GPIO          (0x023<<1)    // r/w  General-purpose I/O
#define S2226_REG_GPREG         (0x024<<1)    // r/w  Test register for datapath integrety test

// Audio Level meter
#define S2226_REG_AUDLVLMTRCTRL (0x025<<1)    // r/w  Audio Level Meter Control
#define S2226_REG_MTR_AUDL0     (0x026<<1)    // Peak and Decayed Left   Audio Amplitude LSB
#define S2226_REG_MTR_AUDL1     (0x027<<1)    // MSB ( 6 :  0) = AMTR_AUDL(22 : 16)
#define S2226_REG_MTR_AUDR0     (0x028<<1)    // Peak and Decayed Right  Audio Amplitude LSB
#define S2226_REG_MTR_AUDR1     (0x029<<1)    // MSB ( 6 :  0) = AMTR_AUDR(22 : 16)
#define S2226_REG_MTR_DB_L      (0x02A<<1)    // Left  Decibel level, 11 bit, unsigned binary 0=max volume (11-bits) 2048 steps, -0.1 db each.
#define S2226_REG_MTR_DB_R      (0x02B<<1)    // Right Decibel level, 11 bit, unsigned binary 0=max volume (11-bits) 2048 steps, -0.1 db each.
#define S2226_REG_MTR_HLDL      (0x02C<<1)    // Left Clip detect and hold value. 
                                              //   MSB=1 indicates (0x7FFFFF or 0x800000 detected). 
                                              //   Bits(10:0) Left Held Decibel level, 11 bit, unsigned binary 0=max volume (11-bits) 2048 steps, -0.1 db each.
#define S2226_REG_MTR_HLDR      (0x02D<<1)    // Right Clip detect and hold value. 
                                              //   MSB=1 indicates (0x7FFFFF or 0x800000 detected). 
                                              //   Bits(10:0) Right Held Decibel level, 11 bit, unsigned binary 0=max volume (11-bits) 2048 steps, -0.1 db each.

// Audio Level Meter Control - Bits
#define S2226_ALMC_HOLD_TIM     0x00E0  // Bits(7:5); Hold time. 0=no hold, 7=hold forever, else count of 1/2 sec's to hold highest value (3-bits)
#define S2226_ALMC_HOLD_0       0x0000  // 0=no hold (Update every 1 ms)
#define S2226_ALMC_HOLD_1       0x0020  // Hold hold highest value 0.5 sec
#define S2226_ALMC_HOLD_2       0x0040  // Hold hold highest value 1.0 sec
#define S2226_ALMC_HOLD_3       0x0060  // Hold hold highest value 1.5 sec
#define S2226_ALMC_HOLD_4       0x0080  // Hold hold highest value 2.0 sec
#define S2226_ALMC_HOLD_5       0x00A0  // Hold hold highest value 2.5 sec
#define S2226_ALMC_HOLD_6       0x00D0  // Hold hold highest value 3.0 sec
#define S2226_ALMC_HOLD_7       0x00E0  // 7=hold forever
#define S2226_ALMC_HOLD_RLS     0x0010  // Force release of 'Held' output
#define S2226_ALMC_TST          0x000C  // Bits(3:@) Audio Test 00=Audio In, 01=Force to zero, 10=Force to clip, 11=Force to -6dB (2-bits)
#define S2226_ALMC_TST_NONE     0x0000  // No Test. 
#define S2226_ALMC_TST_1        0x0004  // 01=Force to zero
#define S2226_ALMC_TST_2        0x0008  // 10=Force to clip
#define S2226_ALMC_TST_3        0x000C  // 11=Force to -6dB
#define S2226_ALMC_CHNSEL       0x0003  // Bits(1:0) Select which channel to meter (2-bits)
#define S2226_ALMC_CHNSEL_0     0x0000  // Channel 0 = Line-in
#define S2226_ALMC_CHNSEL_1     0x0001  // Channel 1 = Mpeg-out
#define S2226_ALMC_CHNSEL_2     0x0002  // Channel 2 = SDI-in
#define S2226_ALMC_CHNSEL_3     0x0003  // Channel 3


// Stream FIFO (Between Fujitsu H51 codec and ARM cpu). 
// FIFO capacity is 1KB, accessed as 512 16-bit words.
#define S2226_REG_FIFO_MODE     (0x030<<1)
#define S2226_REG_FIFO_LOWATR   (0x031<<1)
#define S2226_REG_FIFO_HIWATR   (0x032<<1)
#define S2226_REG_FIFO_STATUS   (0x033<<1)
#define S2226_REG_FIFO_PKTSZ    (0x034<<1)
#define S2226_REG_FIFO_TMSTMP0  (0x035<<1)
#define S2226_REG_FIFO_TMSTMP1  (0x036<<1)
#define S2226_REG_FIFO_RDDATA   (0x800<<1) // thru BFF
#define S2226_REG_FIFO_WRDATA   (0xC00<<1) // thru FFF

// Video offset and gain.
#define S2226_REG_CONTRAST      (0x044<<1)    // w    contrast
#define S2226_REG_BRIGHTNESS    (0x045<<1)    // w    brightness
#define S2226_REG_CRSAT         (0x046<<1)    // w    red saturation
#define S2226_REG_CBSAT         (0x047<<1)    // w    blue saturation

// Overlay window configuration (indirect access via index register).
#define S2226_REG_WCDATA        (0x050<<1)    // w    Write Data to WindowConfig[WCINDEX]
#define S2226_REG_WCINDEX       (0x051<<1)    // w    Index/Address into Window Configuration Registers

// Overlay memory.
#define S2226_REG_OVDATA        (0x052<<1)    // w    Write 8-bit overlay data, auto-inc address after two writes = 16-bits
#define S2226_REG_OVACLR        (0x053<<1)    // w    reset overlay address(mar) to zero

#define S2226_REG_OVALSB        (0x054<<1)    // w    mar LSB
#define S2226_REG_OVAMSB        (0x055<<1)    // w    mar MSB
#define S2226_REG_OVAUPD        (0x056<<1)    // w    Set LSB to update the overlay
#define S2226_REG_OVAMSB2       (0x057<<1)    // w    V.1.3.5 Top 6 bits of write address. V.1.3.6+ Top 7 bits of write address
#define S2226_REG_OVASTAT       (0x058<<1)    // r    Read bank bits. Overlay is being read from (B1) and written to (B0)
#define S2226_REG_OVDATA16      (0x05A<<1)    // w    Write 16-bit overlay data, auto-inc address after each write
#define S2226_REG_OVDATA_RANGE (0x20000<<1)    // thru 0x2FFFF. FPGA maps each address to S2226_REG_OVDATA16

// Snapshot control
#define S2226_REG_FREEZE        (0x060<<1)    // w    Lock output image until cleared
#define S2226_REG_SNAPREQ       (0x061<<1)    // w    Transfer image to host
#define S2226_REG_SNAPACK       (0x062<<1)    // r    Image data is valid (In response to req)
#define S2226_REG_SNAPDAT       (0x063<<1)    // r    Image data. Consecutive access read LSW and MSB of captured pixes. First access reads CrCb
                                              //      SNAPSTAT(0) Toggles each time SNAPDAT read. 
                                              //        When SNAPSTAT(0)=0 - MSB=Cr(7..0), LSB=Cb(7..0) will be read on next SNAPDAT access.
                                              //        When SNAPSTAT(0)=1 - Y(7..0) will be read on LSB of next SNAPDAT access.
#define S2226_REG_SNAPSTAT      (0x064<<1)    // r    Reset image data sequencer, Returns SNAPDAT read type

#define S2226_REG_VRAWSTAT      (0x070<<1)    // r    Bit-0: STREAM_FAV (Frame available when high)
#define S2226_REG_VRAWREAD      (0x071<<1)    // r    Raw Video in 422 format (1st word Y0:Cb0, 2nd word Y1:Cr0)
#define S2226_REG_VRAWCTL       (0x072<<1)    // w    Bit-2: 1=Reset, Bit-1,0: Format. Bit-1 swap MSB/LSB. Bit-0 swap color.

// video to D1 scalar(MPEG) reset must be active when registers written
#define S2226_REG_MPEGHORZ0     (0x80<<1)    // w    SourceWidth LSB
#define S2226_REG_MPEGHORZ1     (0x81<<1)    // w    SourceWidth MSB
#define S2226_REG_MPEGHORZ2     (0x82<<1)    // w    ScaledWidth LSB
#define S2226_REG_MPEGHORZ3     (0x83<<1)    // w    ScaledWidth MSB
#define S2226_REG_MPEGHORZ4     (0x84<<1)    // w    131072 / ScaledWidth LSB
#define S2226_REG_MPEGHORZ5     (0x85<<1)    // w    131072 / ScaledWidth MSB
#define S2226_REG_MPEGHORZ6     (0x86<<1)    // w    (int) (SourceWidth / ScaledWidth)
#define S2226_REG_MPEGHORZ7     (0x87<<1)    // w    (SourceWidth % ScaledWidth) LSB
#define S2226_REG_MPEGHORZ8     (0x88<<1)    // w    (SourceWidth % ScaledWidth) MSB
#define S2226_REG_MPEGHORZ9     (0x89<<1)    // w    Start active scaled width
#define S2226_REG_MPEGHORZ10    (0x8A<<1)    // w    Start active scaled width
#define S2226_REG_MPEGHORZ11    (0x8B<<1)    // w    End active scaled width LSB
#define S2226_REG_MPEGHORZ12    (0x8C<<1)    // w    End active scaled width
#define S2226_REG_MPEGHORZ13    (0x8D<<1)    // w    NegFramesz = Two's complement(FrameSize pixels minus 1)
#define S2226_REG_MPEGHORZ14    (0x8E<<1)    // w    NegFramesz middle byte
#define S2226_REG_MPEGHORZ15    (0x8F<<1)    // w    NegFramesz MSB (20-bits)

#define S2226_REG_MPEGVERT0     (0x90<<1)    // w    SourceHeight LSB
#define S2226_REG_MPEGVERT1     (0x91<<1)    // w    SourceHeight MSB
#define S2226_REG_MPEGVERT2     (0x92<<1)    // w    ScaledHeight LSB
#define S2226_REG_MPEGVERT3     (0x93<<1)    // w    ScaledHeight MSB
#define S2226_REG_MPEGVERT4     (0x94<<1)    // w    65536 / ScaledHeight LSB
#define S2226_REG_MPEGVERT5     (0x95<<1)    // w    65536 / ScaledHeight MSB
#define S2226_REG_MPEGVERT6     (0x96<<1)    // w    (int) (SourceHeight / ScaledHeight)
#define S2226_REG_MPEGVERT7     (0x97<<1)    // w    SourceHeight % ScaledHeight LSB
#define S2226_REG_MPEGVERT8     (0x98<<1)    // w    SourceHeight % ScaledHeight MSB
#define S2226_REG_MPEGVERT9     (0x99<<1)    // w    activestartY( 7 downto 0)
#define S2226_REG_MPEGVERT10    (0x9A<<1)    // w    activestartY((CBITS-0) downto 8)
#define S2226_REG_MPEGVERT11    (0x9B<<1)    // w    activeendY( 7 downto 0)
#define S2226_REG_MPEGVERT12    (0x9C<<1)    // w    activeendY((CBITS-0) downto 8)
#define S2226_REG_MPEGVERT13    (0x9D<<1)    // w    negbframesz( 7 downto  0) <= HOST_DATA(7 downto 0);   - Negative frame size in bursts minus 1 (3 bytes per pixel, 256 bytes per burst)
#define S2226_REG_MPEGVERT14    (0x9E<<1)    // w    negbframesz(11 downto  8) <= HOST_DATA(3 downto 0);   -  :
#define S2226_REG_MPEGVERT15    (0x9F<<1)    // w    SkipField                 <= HOST_DATA(0) - Skip data from Field 1 (second field)

#define S2226_REG_MPEGCALC0     (0xA0<<1)    // w    croppedX LSB
#define S2226_REG_MPEGCALC1     (0xA1<<1)    // w    croppedX MSB
#define S2226_REG_MPEGCALC2     (0xA2<<1)    // w    croppedX_minus2 LSB
#define S2226_REG_MPEGCALC3     (0xA3<<1)    // w    croppedX_minus2 MSB
#define S2226_REG_MPEGCALC4     (0xA4<<1)    // w    ratioY_int_minus1
#define S2226_REG_MPEGCALC5     (0xA5<<1)    // w    ratioX_rem_diff LSB
#define S2226_REG_MPEGCALC6     (0xA6<<1)    // w    ratioX_rem_diff MSB
#define S2226_REG_MPEGCALC7     (0xA7<<1)    // w     LSB
#define S2226_REG_MPEGCALC8     (0xA8<<1)    // w     MSB
#define S2226_REG_MPEGCALC9     (0xA9<<1)    // w    
#define S2226_REG_MPEGCALC10    (0xAA<<1)    // w    
#define S2226_REG_MPEGCALC11    (0xAB<<1)    // w    
#define S2226_REG_MPEGCALC12    (0xAC<<1)    // w    
#define S2226_REG_MPEGCALC13    (0xAD<<1)    // w    
#define S2226_REG_MPEGCALC14    (0xAE<<1)    // w    
#define S2226_REG_MPEGCALC15    (0xAF<<1)    // w    

#define S2226_REG_PLLFLAGS      (0xE6<<1)    // r    [15..6]=errors(9 downto 0), 0, MEMCHECK_DONE, PLLSTATUS[3..0]

// Bits in S2226_REG_PLLFLAGS
#define S2226_BIT_STREAM_SAV_N  0x8000
#define S2226_BIT_STREAM_ERROR  0x4000
#define S2226_BIT_STREAM_DAV_N  0x2000
#define S2226_BIT_SNAP_ERROR1   0x1000
#define S2226_BIT_SNAP_ERROR2   0x0800
#define S2226_BIT_SNAP_DAV_N    0x0400
#define S2226_BIT_SOURCESAV_N   0x0080
#define S2226_BIT_MEMCHECK_DONE 0x0010

// Bits in S2226_REG_DEBUG
#define S2226_BIT_VID_SWAP      0x8000 // Switch MPEG out and Encoder out for debug purposes.
#define S2226_BIT_BYTE_SWAP     0x4000 // Switch MPEG out MSB/LSB for debug purposes.
#define S2226_BIT_RESET_ERR     0x0001 // Clear upper 10-bits (error bits) in C_ADDR_PLLSTAT register

// Bits in S2226_REG_VRAWSTAT
#define S2226_BIT_PFPIX         0x0008 // 1 = Next read of VRAWREAD is First Pixel of Frame
#define S2226_BIT_PLPIX         0x0004 // 1 = Next read of VRAWREAD is Last Pixel of Frame
#define S2226_BIT_PMTY          0x0002 // 1 = Fifo feeding VRAWREAD is empty
#define S2226_BIT_PFAV          0x0001 // Frame available when high
// Bits in S2226_REG_VRAWCTL
#define S2226_BIT_PFMT_422      0x0000
#define S2226_BIT_PFMT_SCLR     0x0001 // 422 Swap color
#define S2226_BIT_PFMT_SBYTE    0x0002 // 422 Swap MSB/LSB
#define S2226_BIT_PFMT_SBOTH    0x0003 // 422 Swap color and MSB/LSB
#define S2226_BIT_RESET_PREV    0x0004
#define S2226_BIT_DISPLAY       0x0010 // Send Scaled video(preview) to Vid out Encoder
#define S2226_BIT_INTERLACE     0x0020 // Interlace Field1 and Field2 before sending out.

// Bits in FPGA_REG_SDIIN/SDIOUT -----------------
#define SDI_DRV					0x2000				// Enable for AD[9:0] tri-state drivers.
#define SDI_ACLK				0x1000				// Clock. Controls data flow through port. Must have freq lower than VCLK.
#define SDI_ANC					0x0800				// Ancillary fifo access (vs. control register access). This version always asserts this signal low.
#define SDI_RD					0x0400
#define SDI_AD9					0x0200
#define SDI_AD8					0x0100
// Derived:
#define SDI_BUS_DRIVE			SDI_DRV
#define SDI_BUS_FLOAT			0
#define SDI_CLK_HI				SDI_ACLK
#define SDI_CLK_LO				0
#define SDI_READ				SDI_RD
#define SDI_WRITE				0
#define SDI_PHASE_ADDR			0
#define SDI_PHASE_DATA			( SDI_AD9 | SDI_AD8 )

//////////////////////////////////////////////////////////////////////////
// Indirect access registers
//////////////////////////////////////////////////////////////////////////

// These are write-only registers, accessed indirectly via S2226_REG_WCDATA and S2226_REG_WCINDEX.

// Window boundary register base addresses (Indexed address space)
// Add (8*WINDOW) to base address for a specific window, where WINDOW is in range 0:7.
#define S2226_REG_LBLSB         0x00    // Window beginning line LSB
#define S2226_REG_LBMSB         0x01    // Window beginning line MSB
#define S2226_REG_LELSB         0x02    // Window ending line LSB
#define S2226_REG_LEMSB         0x03    // Window ending line MSB
#define S2226_REG_CBLSB         0x04    // Window beginning column LSB
#define S2226_REG_CBMSB         0x05    // Window beginning column MSB
#define S2226_REG_CELSB         0x06    // Window ending column LSB
#define S2226_REG_CEMSB         0x07    // Window ending column MSB

// Black window boundary register base addresses (Indexed address space)
// Add (8*WINDOW) to base address for a specific window, where WINDOW is in range 0:1.
#define S2226_REG_BWLBLSB       0x80    // Black window beginning line LSB
#define S2226_REG_BWLBMSB       0x81    // Black window beginning line MSB
#define S2226_REG_BWLELSB       0x82    // Black window ending line LSB
#define S2226_REG_BWLEMSB       0x83    // Black window ending line MSB
#define S2226_REG_BWCBLSB       0x84    // Black window beginning column LSB
#define S2226_REG_BWCBMSB       0x85    // Black window beginning column MSB
#define S2226_REG_BWCELSB       0x86    // Black window ending column LSB
#define S2226_REG_BWCEMSB       0x87    // Black window ending column MSB

// Black window color (Indexed address space)
#define S2226_REG_BWCOLOR0      0xC0    // Black window color LSB
#define S2226_REG_BWCOLOR1      0xC1    // Black window color MSB

// Bit mask for addressing Overlay memory
#define S2226_MSK_OVLMEM        0x7FFFFF // Bits covering max addressable range in overlay

// Registers in SDI IC
#define ADDRESS_SDI_FORMAT0     0x0B
#define ADDRESS_SDIIN_FORMAT1   0x0C
#define ADDRESS_SDIIN_TEST      0x0D
#define ADDRESS_SDIIN_VIDCTL0   0x55

void FPGAOpen();
void FPGAClose();

// WriteIndexByte and some other routines must be protected 
// by a critical section(any) to make it atomic.
// Currently, these calls are protected by m_csOvl in mid2226.cpp

void WriteIndexedByte( MYHDEVICE handle, int addr, int value );
void SetOverlayWindowPosition( MYHDEVICE m_handle, int window_num, int regionmask, int top, int bottom, int left, int right );
void SetBlackWindowPosition( MYHDEVICE m_handle, int window_num, int regionmask, int top, int bottom, int left, int right );
void SetBlackWindowColor( MYHDEVICE m_handle, int color );
//void SetContrast( MYHDEVICE m_handle, int level );  
//void SetBrightness( MYHDEVICE m_handle, int level ); 
//void SetCrSaturation( MYHDEVICE m_handle, int level );
//void SetCbSaturation( MYHDEVICE m_handle, int level );
//void SetCrHue( MYHDEVICE m_handle, int level );
//void SetCbHue( MYHDEVICE m_handle, int level );
int  GetSnapshot( MYHDEVICE m_handle, void *buf, int *bytecount, int freezetime, int wait );
void SetPreviewReset( MYHDEVICE handle, int bReset);
int  GetPreview( MYHDEVICE handle, void *buf, int bytecount);
int  SetOverlayMem( MYHDEVICE m_handle, void *buf, int *bytecount, int wait, int arm_fw );
//int  S2226WaitForBist( MYHDEVICE handle);
//int  S2226SetVideoConfig( MYHDEVICE handle, MID2226_SOURCE video_source, MID2226_VIDSYS video_type );
//void S2226GetVideoConfig( MYHDEVICE m_handle, int *video_source, int *video_type );
//void SetGpouts( MYHDEVICE m_handle, int states );
//void GetGpouts( MYHDEVICE m_handle, int *states );
//void GetGpins( MYHDEVICE m_handle, int *states );
//void SetLeds( MYHDEVICE m_handle, int states );
//void GetLeds( MYHDEVICE m_handle, int *states );
//
//void GetErrorFlags( MYHDEVICE m_handle, int *errflags );
//void GetFpgaVersion( MYHDEVICE m_handle, int *version );

int SetMpegScaler(MYHDEVICE handle, int input, int ScaleToX, int ScaleToY, int bSingleField, int bInterlace, int CropLeft, int CropRight);

// primes the SDI ports
void sdi_init( MYHDEVICE handle);
// Read SDI IN.
void SdiInRead( MYHDEVICE handle, int addr, unsigned short *value );
// Write SDI IN.
void SdiInWrite( MYHDEVICE handle, int addr, unsigned short value );
// Read SDI OUT.
void SdiOutRead( MYHDEVICE handle, int addr, unsigned short *value );
// Write SDI OUT.
void SdiOutWrite( MYHDEVICE handle, int addr, unsigned short value );

//
void WriteAdrByteMem( char *pmem, int *pos, char addr, char value );
void WriteIndexedByteMem( char *pmem, int *pos, int addr, int value );
void SetOverlayWindowPositionMem( char *pmem, int *pos, int window_num, int regionmask, int top, int bottom, int left, int right );
void SetBlackWindowPositionMem( char *pmem, int *pos, int window_num, int regionmask, int top, int bottom, int left, int right );
void SetBlackWindowColorMem( char *pmem, int *pos, int color );

//////////////////////////////////////////////////////////////////////////
// Draw pixel with caching
//////////////////////////////////////////////////////////////////////////

void plotInit(int aBank, int aDDRImageWidth, int aDDROffsetToField1); // Initalize the global plotting variables
//void plotPoint(MYHDEVICE handle, int x, int y, int clr); // Plot the pixel at (x, y) with brightness c (where 0 = c = 1)
void plotPoint(MYHDEVICE handle, char *pmem, int *pos, int x, int y, int clr); // Plot the pixel at (x, y) with brightness c (where 0 = c = 1)
//void plotFlush(MYHDEVICE handle); // Make sure the last few pixels get written to the overlay memory
void plotFlush(MYHDEVICE handle, char *pmem, int *pos); // Make sure the last few pixels get written to the overlay memory

void drawLine(MYHDEVICE handle, unsigned char grey, int x0, int y0, int x1, int y1); // Standard Bresenham line drawing routine

#endif // _FPGA2226_H
